==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-workflow/src/main/java/de/fzj/unicore/ucc/workflow/SubmitWorkflow.java <==
package de.fzj.unicore.ucc.workflow;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Random;

import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlCursor.TokenType;
import org.chemomentum.common.api.workflow.WorkflowWrapper;
import org.chemomentum.common.impl.workflow.WorkflowManagementClient;
import org.chemomentum.common.impl.workflow.WorkflowSubmissionClient;
import org.chemomentum.common.ws.IWorkflowFactory;
import org.chemomentum.workflow.xmlbeans.RulesDocument;
import org.chemomentum.workflow.xmlbeans.StatusType;
import org.chemomentum.workflow.xmlbeans.WorkflowDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDescriptionDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.actions.Resolve;
import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.util.Builder;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import eu.unicore.security.util.client.IClientProperties;

/**
 * submit a workflow to the UNICORE workflow engine
 * 
 * @author schuller
 */
public class SubmitWorkflow extends WorkflowSystemSubmission implements
		IServiceInfoProvider {

	public static final String OPT_NAME_LONG = "workflowName";
	public static final String OPT_NAME = "N";

	public static final String OPT_UFILE_LONG = "uccInput";
	public static final String OPT_UFILE = "u";

	public static final String OPT_LIFETIME = "t";
	public static final String OPT_LIFETIME_LONG = "lifetime";

	public static final String OPT_WAIT = "w";
	public static final String OPT_WAIT_LONG = "wait";

	protected String workflowName;

	protected WorkflowSubmissionClient wsc;

	protected String workflowToBeSubmitted;

	protected StatusType.Enum status;

	protected String workflowFileName;

	protected boolean wait = false;

	// initialise the workflow support stuff
	static {
		Resolve.addResolver("c9m:",
				new LocationResolver());
	}

	@Override
	public void process() {
		super.process();

		workflowName = getCommandLine().getOptionValue(OPT_NAME);
		if (getCommandLine().hasOption(OPT_WAIT)) {
			wait = true;
		}

		if (getCommandLine().getArgs().length < 2) {
			error("Please supply the name of the workflow file,",
					new IllegalArgumentException());
			endProcessing(1);
		} else {
			workflowFileName = getCommandLine().getArgs()[1];
		}

		try {
			findSite();
			createBuilder();
			createWorkflowDataStorage();
			run();
		} catch (Exception e) {
			error("", e);
			endProcessing(1);
		}

	}

	protected void createBuilder()throws Exception{
		String uFile = getOption(OPT_UFILE_LONG, OPT_UFILE);
		if (uFile != null) {
			verbose("Reading stage-in definitions from <" + uFile + ">");
			builder = new Builder(new File(uFile));
			//side effect: existence of local files will be checked
			int n=builder.getImports().size();
			verbose("Will upload <" + n + "> files");
		}
	}
	
	protected void findSite() {
		try {
			List<EndpointReferenceType> available = registry
					.listAccessibleServices(IWorkflowFactory.PORTTYPE);
			for (EndpointReferenceType epr : available) {
				if (siteName != null) {
					String addr = epr.getAddress().getStringValue();
					if (addr.contains(siteName)) {
						wsc = new WorkflowSubmissionClient(addr, epr,
								securityProperties);
						verbose("Using service <" + addr
								+ "> at requested site <" + siteName + ">");
						return;
					}
				}
			}
			int i = 0;
			if (available.size() == 0) {
				message("No workflow service available!");
				endProcessing(1);
			} else {
				// select one
				i = new Random().nextInt(available.size());
				EndpointReferenceType epr = available.get(i);
				wsc = new WorkflowSubmissionClient(epr.getAddress()
						.getStringValue(), epr, securityProperties);
			}
			verbose("Selected workflow service at "
					+ wsc.getEPR().getAddress().getStringValue());
		} catch (Exception e) {
			error("Can't find a workflow service.", e);
			endProcessing(1);
		}
	}

	protected void run() throws Exception {
		loadWorkflowFromFile();

		Calendar terminationTime = Calendar.getInstance();
		terminationTime.add(Calendar.HOUR, lifetime);
		verbose("Setting initial termination time to "
				+ terminationTime.getTime());

		String uFile = getOption(OPT_UFILE_LONG, OPT_UFILE);
		if (uFile != null) {
			verbose("Reading stage-in definitions from <" + uFile + ">");
			builder = new Builder(new File(uFile));
			//side effect: existence of local files will be checked
			int n=builder.getImports().size();
			verbose("Will upload <" + n + "> files");
		}

		WorkflowManagementClient wmc = wsc.createWorkflow(workflowName,
				terminationTime, storageEPR);
		EndpointReferenceType epr = wmc.getEPR();
		String id = WSUtilities.extractResourceID(epr);
		
		if (fixFilenames) {
			// make global file IDs unique to allow re-submission
			// from the same workflow file
			workflowToBeSubmitted = writeNewIDs(workflowToBeSubmitted, id);
		}
		workflowToBeSubmitted = resolveU6URLs(workflowToBeSubmitted);
		verbose("Workflow instance EPR: " + epr.getAddress().getStringValue());

		final WorkflowDocument wf = load();

		boolean validWF=isValid(wf);
		if (!validWF){
			boolean failOnError=Boolean.parseBoolean(properties.getProperty(PROP_FAIL_ON_VALIDATION_ERRORS));
			if(failOnError){
				String message = "Stopping due to validation errors (set property '"
						+ PROP_FAIL_ON_VALIDATION_ERRORS+"=false' to ignore them)";
				throw new EndProcessingException(ERROR_JSDL_INVALID, message);
			}
			else{
				verbose("Ignoring validation errors because property "+
						PROP_FAIL_ON_VALIDATION_ERRORS+" is set to 'false'.");
			}
		}

		final RulesDocument rd = RulesDocument.Factory.newInstance();
		rd.addNewRules();

		if(builder!=null){
			try {
				uploadLocalData(builder, id);
			} catch (IOException e) {
				error("Can't upload local files.", e);
				endProcessing(1);
			}
		}
		
		WorkflowWrapper w = new WorkflowWrapper(wf.getWorkflow(), rd.getRules());
		w.setTerminationTime(terminationTime);
		verbose("Submitting...");
		wmc.submitWorkflow(w);
		writeIDFile(epr);

		// output this to allow nicer shell scripting
		message(epr.getAddress().getStringValue());

		if (wait) {
			verbose("Waiting for workflow to finish...");
			do {
				Thread.sleep(2000);
				StatusType.Enum newStatus = wmc.getStatus();
				if (newStatus == null) {
					error("Can't get workflow status.", null);
					break;
				}
				if (!newStatus.equals(status)) {
					status = newStatus;
					verbose(status.toString());
				}
			} while (StatusType.RUNNING.equals(status));
		}

	}

	WorkflowDocument load() throws Exception {
		WorkflowDocument wf = WorkflowDocument.Factory.newInstance();
		String dialect = null;
		XmlObject x = XmlObject.Factory.parse(workflowToBeSubmitted);
		wf.addNewWorkflow().set(x);
		// get the namespace of the root element
		dialect = x.newDomNode().getFirstChild().getNamespaceURI();
		wf.getWorkflow().setDialect(dialect);
		verbose("Workflow dialect '" + dialect + "'");
		return wf;
	}

	void loadWorkflowFromFile() throws Exception {
		FileInputStream fis = new FileInputStream(new File(workflowFileName)
				.getAbsolutePath());
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			int b = 0;
			while ((b = fis.read()) != -1) {
				bos.write(b);
			}
			workflowToBeSubmitted = bos.toString();
		} finally {
			fis.close();
		}
	}

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();

		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_NAME_LONG).withDescription(
						"Workflow Name").withArgName("Name").hasArg()
						.isRequired(false).create(OPT_NAME));

		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_UFILE_LONG).withDescription(
						"UCC .u file with stage-in definitions").isRequired(
						false).hasArg().create(OPT_UFILE));

		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_WAIT_LONG).withDescription(
						"wait for workflow completion").isRequired(false)
						.create(OPT_WAIT));

	}

	protected void writeIDFile(EndpointReferenceType epr) {
		try {
			File dump = new File(output, WSUtilities.extractResourceID(epr)
					+ ".workflow-address");
			FileWriter fw = new FileWriter(dump);
			fw.write(epr.toString() + "\n");
			fw.close();
			verbose("Wrote workflow address to " + dump.getAbsolutePath());
		} catch (Exception e) {
			error("Could not write workflow ID file.", e);
		}
	}

	@Override
	public String getName() {
		return "workflow-submit";
	}

	@Override
	public String getArgumentList() {
		return "[<workflow-file>]";
	}

	@Override
	public String getSynopsis() {
		return "Runs a workflow. "
				+ "The workflow definition is read from <workflow-file>."
				+ "A descriptor file will be written that can be used "
				+ "later with other ucc commands.";
	}

	@Override
	public String getDescription() {
		return "submit a workflow";
	}

	@Override
	public String getCommandGroup() {
		return "Workflow";
	}

	public QName getQName() {
		return IWorkflowFactory.PORTTYPE;
	}

	public String getServiceName() {
		return "Workflow submission";
	}

	public String getServiceDetails(EndpointReferenceType epr,
			IClientProperties security) {
		try {
			WorkflowSubmissionClient wsc = new WorkflowSubmissionClient(epr
					.getAddress().getStringValue(), epr, security);
			return "Version: "
					+ wsc.getResourcePropertiesDocument()
							.getWorkflowFactoryProperties().getVersion();
		} catch (Exception ex) {
			error("Can't get workflow service details", ex);
		}
		return null;
	}

	protected boolean isValid(WorkflowDocument workflow) throws XmlException {
		boolean isValid = true;
		List<JobDescriptionDocument> jsdls = extractAnyElements(workflow,
				JobDescriptionDocument.type.getDocumentElementName(),
				JobDescriptionDocument.class);
		verbose("Validating " + jsdls.size() + " JSDL job definitions.");
		for (XmlObject j : jsdls) {
			try {
				JobDefinitionDocument jdd = JobDefinitionDocument.Factory
						.newInstance();
				JobDescriptionDocument jdesc = (JobDescriptionDocument) j;
				jdd.addNewJobDefinition().setJobDescription(
						jdesc.getJobDescription());
				boolean isValidPart = Builder.isValidJSDL(jdd, this);
				isValid = isValid & isValidPart;
			} catch (Exception ex) {
				error("", ex);
			}
		}
		return isValid;
	}

	/**
	 * TODO move to some better place extract all XML elements identified by
	 * their qname from an XML source document.
	 * 
	 * @param source
	 *            - the xml fragment
	 * @param q
	 *            - QName of elements to extract
	 * @param asClass
	 *            - the XMLbeans class of the elements to extract
	 * @return List<T> - a list of XMLBeans objects with the correct runtime
	 *         type
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> extractAnyElements(XmlObject source, QName q,
			Class<? extends XmlObject> asClass) {
		List<T> results = new ArrayList<T>();
		XmlCursor cursor = null;
		String ns = q.getNamespaceURI();
		try {
			if (source != null) {
				cursor = source.newCursor();
				while (skipToElement(cursor, q)) {
					XmlObject next = XmlObject.Factory.parse(cursor
							.newXMLStreamReader());
					QName name = cursor.getName();
					if ((name != null && (ns.equals(name.getNamespaceURI()) || "*"
							.equals(ns)))
							&& q.getLocalPart().equals(name.getLocalPart())) {
						results.add((T) next);
					}
				}
			}
		} catch (XmlException xe) {
			// what?
		} finally {
			if (cursor != null)
				cursor.dispose();
		}
		return results;
	}

	public static boolean skipToElement(XmlCursor cursor, QName name) {
		while (cursor.hasNextToken()) {
			TokenType tt = cursor.toNextToken();
			if (tt.isStart()) {
				if (name.equals(cursor.getName())) {
					return true;
				}
			}
		}
		return false;
	}

}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-workflow/src/main/java/de/fzj/unicore/ucc/workflow/SubmitWorkflow.java <==
package de.fzj.unicore.ucc.workflow;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Random;

import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlCursor.TokenType;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.chemomentum.common.api.workflow.WorkflowWrapper;
import org.chemomentum.common.impl.workflow.WorkflowManagementClient;
import org.chemomentum.common.impl.workflow.WorkflowSubmissionClient;
import org.chemomentum.common.ws.IWorkflowFactory;
import org.chemomentum.workflow.xmlbeans.RulesDocument;
import org.chemomentum.workflow.xmlbeans.StatusType;
import org.chemomentum.workflow.xmlbeans.WorkflowDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDescriptionDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.IServiceInfoProvider;
import de.fzj.unicore.ucc.actions.Resolve;
import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.util.Builder;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * submit a workflow to the UNICORE workflow engine
 * 
 * @author schuller
 */
public class SubmitWorkflow extends WorkflowSystemSubmission implements
		IServiceInfoProvider {

	public static final String OPT_NAME_LONG = "workflowName";
	public static final String OPT_NAME = "N";

	public static final String OPT_UFILE_LONG = "uccInput";
	public static final String OPT_UFILE = "u";

	public static final String OPT_LIFETIME = "t";
	public static final String OPT_LIFETIME_LONG = "lifetime";

	public static final String OPT_WAIT = "w";
	public static final String OPT_WAIT_LONG = "wait";

	protected String workflowName;

	protected WorkflowSubmissionClient wsc;

	protected String workflowToBeSubmitted;

	protected StatusType.Enum status;

	protected String workflowFileName;

	protected boolean wait = false;

	// initialise the workflow support stuff
	static {
		Resolve.addResolver("c9m:",
				new LocationResolver());
	}

	@Override
	public void process() {
		super.process();

		workflowName = getCommandLine().getOptionValue(OPT_NAME);
		if (getCommandLine().hasOption(OPT_WAIT)) {
			wait = true;
		}

		if (getCommandLine().getArgs().length < 2) {
			error("Please supply the name of the workflow file,",
					new IllegalArgumentException());
			endProcessing(1);
		} else {
			workflowFileName = getCommandLine().getArgs()[1];
		}

		try {
			findSite();
			createBuilder();
			createWorkflowDataStorage();
			run();
		} catch (Exception e) {
			error("", e);
			endProcessing(1);
		}

	}

	protected void createBuilder()throws Exception{
		String uFile = getOption(OPT_UFILE_LONG, OPT_UFILE);
		if (uFile != null) {
			verbose("Reading stage-in definitions from <" + uFile + ">");
			builder = new Builder(new File(uFile));
			//side effect: existence of local files will be checked
			int n=builder.getImports().size();
			verbose("Will upload <" + n + "> files");
		}
	}
	
	protected void findSite() {
		try {
			List<EndpointReferenceType> available = registry
					.listAccessibleServices(IWorkflowFactory.PORTTYPE);
			for (EndpointReferenceType epr : available) {
				if (siteName != null) {
					String addr = epr.getAddress().getStringValue();
					if (addr.contains(siteName)) {
						wsc = new WorkflowSubmissionClient(addr, epr,
								securityProperties);
						verbose("Using service <" + addr
								+ "> at requested site <" + siteName + ">");
						return;
					}
				}
			}
			int i = 0;
			if (available.size() == 0) {
				message("No workflow service available!");
				endProcessing(1);
			} else {
				// select one
				i = new Random().nextInt(available.size());
				EndpointReferenceType epr = available.get(i);
				wsc = new WorkflowSubmissionClient(epr.getAddress()
						.getStringValue(), epr, securityProperties);
			}
			verbose("Selected workflow service at "
					+ wsc.getEPR().getAddress().getStringValue());
		} catch (Exception e) {
			error("Can't find a workflow service.", e);
			endProcessing(1);
		}
	}

	protected void run() throws Exception {
		loadWorkflowFromFile();

		Calendar terminationTime = Calendar.getInstance();
		terminationTime.add(Calendar.HOUR, lifetime);
		verbose("Setting initial termination time to "
				+ terminationTime.getTime());

		String uFile = getOption(OPT_UFILE_LONG, OPT_UFILE);
		if (uFile != null) {
			verbose("Reading stage-in definitions from <" + uFile + ">");
			builder = new Builder(new File(uFile));
			//side effect: existence of local files will be checked
			int n=builder.getImports().size();
			verbose("Will upload <" + n + "> files");
		}

		WorkflowManagementClient wmc = wsc.createWorkflow(workflowName,
				terminationTime, storageEPR);
		EndpointReferenceType epr = wmc.getEPR();
		String id = WSUtilities.extractResourceID(epr);
		
		if (fixFilenames) {
			// make global file IDs unique to allow re-submission
			// from the same workflow file
			workflowToBeSubmitted = writeNewIDs(workflowToBeSubmitted, id);
		}
		workflowToBeSubmitted = resolveU6URLs(workflowToBeSubmitted);
		verbose("Workflow instance EPR: " + epr.getAddress().getStringValue());

		final WorkflowDocument wf = load();

		boolean validWF=isValid(wf);
		if (!validWF){
			boolean failOnError=Boolean.parseBoolean(properties.getProperty(PROP_FAIL_ON_VALIDATION_ERRORS));
			if(failOnError){
				String message = "Stopping due to validation errors (set property '"
						+ PROP_FAIL_ON_VALIDATION_ERRORS+"=false' to ignore them)";
				throw new EndProcessingException(ERROR_JSDL_INVALID, message);
			}
			else{
				verbose("Ignoring validation errors because property "+
						PROP_FAIL_ON_VALIDATION_ERRORS+" is set to 'false'.");
			}
		}

		final RulesDocument rd = RulesDocument.Factory.newInstance();
		rd.addNewRules();

		if(builder!=null){
			try {
				uploadLocalData(builder, id);
			} catch (IOException e) {
				error("Can't upload local files.", e);
				endProcessing(1);
			}
		}
		
		WorkflowWrapper w = new WorkflowWrapper(wf.getWorkflow(), rd.getRules());
		w.setTerminationTime(terminationTime);
		verbose("Submitting...");
		wmc.submitWorkflow(w);
		writeIDFile(epr);

		// output this to allow nicer shell scripting
		message(epr.getAddress().getStringValue());

		if (wait) {
			verbose("Waiting for workflow to finish...");
			do {
				Thread.sleep(2000);
				StatusType.Enum newStatus = wmc.getStatus();
				if (newStatus == null) {
					error("Can't get workflow status.", null);
					break;
				}
				if (!newStatus.equals(status)) {
					status = newStatus;
					verbose(status.toString());
				}
			} while (StatusType.RUNNING.equals(status));
		}

	}

	WorkflowDocument load() throws Exception {
		WorkflowDocument wf = WorkflowDocument.Factory.newInstance();
		String dialect = null;
		XmlObject x = XmlObject.Factory.parse(workflowToBeSubmitted);
		wf.addNewWorkflow().set(x);
		// get the namespace of the root element
		dialect = x.newDomNode().getFirstChild().getNamespaceURI();
		wf.getWorkflow().setDialect(dialect);
		verbose("Workflow dialect '" + dialect + "'");
		return wf;
	}

	void loadWorkflowFromFile() throws Exception {
		FileInputStream fis = new FileInputStream(new File(workflowFileName)
				.getAbsolutePath());
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			int b = 0;
			while ((b = fis.read()) != -1) {
				bos.write(b);
			}
			workflowToBeSubmitted = bos.toString();
		} finally {
			fis.close();
		}
	}

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();

		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_NAME_LONG).withDescription(
						"Workflow Name").withArgName("Name").hasArg()
						.isRequired(false).create(OPT_NAME));

		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_UFILE_LONG).withDescription(
						"UCC .u file with stage-in definitions").isRequired(
						false).hasArg().create(OPT_UFILE));

		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_WAIT_LONG).withDescription(
						"wait for workflow completion").isRequired(false)
						.create(OPT_WAIT));

	}

	protected void writeIDFile(EndpointReferenceType epr) {
		try {
			File dump = new File(output, WSUtilities.extractResourceID(epr)
					+ ".workflow-address");
			FileWriter fw = new FileWriter(dump);
			fw.write(epr.toString() + "\n");
			fw.close();
			verbose("Wrote workflow address to " + dump.getAbsolutePath());
		} catch (Exception e) {
			error("Could not write workflow ID file.", e);
		}
	}

	@Override
	public String getName() {
		return "workflow-submit";
	}

	@Override
	public String getArgumentList() {
		return "[<workflow-file>]";
	}

	@Override
	public String getSynopsis() {
		return "Runs a workflow. "
				+ "The workflow definition is read from <workflow-file>."
				+ "A descriptor file will be written that can be used "
				+ "later with other ucc commands.";
	}

	@Override
	public String getDescription() {
		return "submit a workflow";
	}

	@Override
	public String getCommandGroup() {
		return "Workflow";
	}

	@Override
	public QName getQName() {
		return IWorkflowFactory.PORTTYPE;
	}

	@Override
	public String getServiceName() {
		return "Workflow submission";
	}

	@Override
	public String getServiceDetails(EndpointReferenceType epr,
			IClientConfiguration security) {
		try {
			WorkflowSubmissionClient wsc = new WorkflowSubmissionClient(epr
					.getAddress().getStringValue(), epr, security);
			return "Version: "
					+ wsc.getResourcePropertiesDocument()
							.getWorkflowFactoryProperties().getVersion();
		} catch (Exception ex) {
			error("Can't get workflow service details", ex);
		}
		return null;
	}

	protected boolean isValid(WorkflowDocument workflow) throws XmlException {
		boolean isValid = true;
		List<JobDescriptionDocument> jsdls = extractAnyElements(workflow,
				JobDescriptionDocument.type.getDocumentElementName(),
				JobDescriptionDocument.class);
		verbose("Validating " + jsdls.size() + " JSDL job definitions.");
		for (XmlObject j : jsdls) {
			try {
				JobDefinitionDocument jdd = JobDefinitionDocument.Factory
						.newInstance();
				JobDescriptionDocument jdesc = (JobDescriptionDocument) j;
				jdd.addNewJobDefinition().setJobDescription(
						jdesc.getJobDescription());
				boolean isValidPart = Builder.isValidJSDL(jdd, this);
				isValid = isValid & isValidPart;
			} catch (Exception ex) {
				error("", ex);
			}
		}
		return isValid;
	}

	/**
	 * TODO move to some better place extract all XML elements identified by
	 * their qname from an XML source document.
	 * 
	 * @param source
	 *            - the xml fragment
	 * @param q
	 *            - QName of elements to extract
	 * @param asClass
	 *            - the XMLbeans class of the elements to extract
	 * @return List<T> - a list of XMLBeans objects with the correct runtime
	 *         type
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> extractAnyElements(XmlObject source, QName q,
			Class<? extends XmlObject> asClass) {
		List<T> results = new ArrayList<T>();
		XmlCursor cursor = null;
		String ns = q.getNamespaceURI();
		try {
			if (source != null) {
				cursor = source.newCursor();
				while (skipToElement(cursor, q)) {
					XmlObject next = XmlObject.Factory.parse(cursor
							.newXMLStreamReader());
					QName name = cursor.getName();
					if ((name != null && (ns.equals(name.getNamespaceURI()) || "*"
							.equals(ns)))
							&& q.getLocalPart().equals(name.getLocalPart())) {
						results.add((T) next);
					}
				}
			}
		} catch (XmlException xe) {
			// what?
		} finally {
			if (cursor != null)
				cursor.dispose();
		}
		return results;
	}

	public static boolean skipToElement(XmlCursor cursor, QName name) {
		while (cursor.hasNextToken()) {
			TokenType tt = cursor.toNextToken();
			if (tt.isStart()) {
				if (name.equals(cursor.getName())) {
					return true;
				}
			}
		}
		return false;
	}

}
